local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("dailyTooltip", "AceEvent-3.0")
local LDT = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-DailyTooltip")
local dataobj = LibStub("LibDataBroker-1.1"):NewDataObject("tomQuest2Daily", {type = "data source", icon = "Interface\\AddOns\\tomQuest2\\media\\icon"})


--------------------------------------
--- Copy global functions to local ---
--------------------------------------
local pairs = pairs
local format = string.format
local mod, abs = mod, math.abs
local GetTime = GetTime
local ToggleFrame = ToggleFrame
local CreateFrame = CreateFrame
local GetQuestResetTime = GetQuestResetTime
local GetMaxDailyQuests = GetMaxDailyQuests

local remainingTime = 0

local defaults = {
	profile = {
		showNumCompletedDailyQuests = true,
		showMaxDailyQuest = true,
		showRemainingTime = true,
	}
}

local prdb
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("dailyTooltip", defaults, "Default")
	prdb = self.db.profile
end

module.modName = LDT["Daily Quests Tooltip"]

local options
function module:getOptions()
	if not options then
		options = {		
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = function(info) 
				return prdb[info[#info]]
			end,
			set = function(info, value)
				prdb[info[#info]] = value
				remainingTime = abs(GetQuestResetTime())
				module:updateLDBText()
			end,
			args = {
				showNumCompletedDailyQuests = {
					name = LDT["Show number of completed daily quest"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				showMaxDailyQuest = {
					name = LDT["Show maximum daily quests number"],
					type = "toggle",
					width = "full",
					order = 12,
				},
				showRemainingTime = {
					name = LDT["Show remaining time before end of day"],
					type = "toggle",
					width = "full",
					order = 13,
				},
			},
		}
	end
	return options
end

local dailytimer = CreateFrame("Frame")

local dailytip
local counter = 0
local delay = 60
local maxDaily = GetMaxDailyQuests()

function module:OnEnable()
	core.RegisterCallback(self, "updateLDBText")
	remainingTime = abs(GetQuestResetTime())
	module:updateLDBText()
end

function dailytimer:OnUpdate(elapsed)
	counter = counter + elapsed
	if counter >= delay then
		remainingTime = abs(GetQuestResetTime())
		module:updateLDBText()
	end
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

function module:updateLDBText(event)
	if prdb.showNumCompletedDailyQuests then
		if prdb.showMaxDailyQuest then
			if prdb.showRemainingTime then
				dataobj.text = format("|cff%s%s/%s|r |cff%s%d:%02d|r", core:getColorFromCompletionHEX(core.numCompletedDailyQuests/maxDaily), core.numCompletedDailyQuests, maxDaily, core:getColorFromCompletionHEX(remainingTime/86400), remainingTime/3600, mod(remainingTime/60, 60))
			else
				dataobj.text = format("|cff%s%s/%s|r", core:getColorFromCompletionHEX(core.numCompletedDailyQuests/maxDaily), core.numCompletedDailyQuests, maxDaily)
			end
		elseif prdb.showRemainingTime then
			dataobj.text = format("|cff%s%s|r |cff%s%d:%02d|r", core:getColorFromCompletionHEX(core.numCompletedDailyQuests/maxDaily), core.numCompletedDailyQuests, core:getColorFromCompletionHEX(remainingTime/86400), remainingTime/3600, mod(remainingTime/60, 60))
		else
			dataobj.text = format("|cff%s%s|r", core:getColorFromCompletionHEX(core.numCompletedDailyQuests/maxDaily), core.numCompletedDailyQuests)
		end
	elseif prdb.showMaxDailyQuest then
		if prdb.showRemainingTime then
			dataobj.text = format("|cffffffff%s|r |cff%s%d:%02d|r", maxDaily, core:getColorFromCompletionHEX(remainingTime/86400), remainingTime/3600, mod(remainingTime/60, 60))
		else
			dataobj.text = maxDaily
		end	
	elseif prdb.showRemainingTime then
		dataobj.text = format("|cff%s%d:%02d|r", core:getColorFromCompletionHEX(remainingTime/86400), remainingTime/3600, mod(remainingTime/60, 60))
	else
		dataobj.text = ""
	end
end

function dataobj.OnClick(self,button)
	if button == "RightButton" then
		return InterfaceOptionsFrame_OpenToCategory(LibStub("AceConfigDialog-3.0").BlizOptions["tomQuest2"].frame)
	else
		ToggleFrame(QuestLogFrame)
	end
end

dailytimer:SetScript("OnUpdate", dailytimer.OnUpdate)