local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("informations", "AceEvent-3.0")
local LI = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-QuestsInformations")
local LSM = LibStub("LibSharedMedia-3.0")

local defaults = {
	profile = {
		font = "Friz Quadrata TT",
		fontSizeH1 = 16,
		fontSizeH2 = 14,
		fontSize = 12,
		width = 640,
		height = 400,
		point = "CENTER",
		x = 0,
		y = 0,
		color = "ffffd100",
	},
}

local prdb
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("informations", defaults, "Default")
	prdb = self.db.profile
end

module.modName = LI["Quests Informations"]

local options
function module:getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = function(info) 
				return prdb[info[#info]]
			end,
			set = function(info, value)
				prdb[info[#info]] = value
			end,
			args = {
				tooltipheader = {
					type = "header",
					name = LI["Quests log settings"],
					order = 10,
				},
				font = {
					name = LI["Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					width = "full",
					order = 11,
				},
				fontSizeH1 = {
					name = LI["Quest title font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 12,
				},
				fontSizeH2 = {
				name = LI["Headers font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 13,
				},
				fontSize = {
					name = LI["text font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 13,
				},
			},
		}
	end
	return options
end

local COLOR_COPPER = "eda55f"
local COLOR_SILVER = "c7c7cf"
local COLOR_GOLD = "ffd700"
local stack = {}

function module:OnEnable()
	core.RegisterCallback(self, "displayQuestInformations")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

function module:SetFrameTooltip(frame, text)
	frame:SetScript("OnEnter", function(frame, ...)
		GameTooltip:SetOwner(frame, "ANCHOR_TOPLEFT")
		GameTooltip:SetText(text)
		GameTooltip:Show()
	end)
	frame:SetScript("OnLeave", function(frame, ...)
		GameTooltip:Hide()
	end)
end

function module:CreateGUI()
	if tomQuest2Frame then return end
	local tq2frame = CreateFrame("Frame", "tomQuest2Frame", UIParent)
	tinsert(UISpecialFrames, "tomQuest2Frame")
	local fontN = CreateFont("tq2NFont")
	fontN:SetFont(LSM:Fetch("font", prdb.font), prdb.fontSize)
	tq2frame:SetWidth(prdb.width)
	tq2frame:SetHeight(prdb.height)
	tq2frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	tq2frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5}, tile = true, tileSize = 16})
	tq2frame:SetBackdropColor(0.09, 0.09, 0.19, 0.5)
	tq2frame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)

	tq2frame:EnableMouse()
	tq2frame:SetMovable(1)
	tq2frame:SetResizable(1)
	tq2frame:SetMinResize(300, 300)
	tq2frame:SetFrameLevel(0)
	tq2frame:SetClampedToScreen(1)

	tq2frame.resize = CreateFrame("Button", nil, tq2frame)
	tq2frame.resize:SetNormalTexture("Interface\\AddOns\\tomQuest2\\media\\Resize")
	tq2frame.resize:SetHighlightTexture("Interface\\AddOns\\tomQuest2\\media\\Resize")
	tq2frame.resize:SetHeight(12)
	tq2frame.resize:SetWidth(12)
	tq2frame.resize:SetPoint("BOTTOMRIGHT", -3, 3)
	module:SetFrameTooltip(tq2frame.resize, LI["Click to resize tomQuest2"])

	tq2frame.titlereg = CreateFrame("Button", nil, tq2frame)
	tq2frame.titlereg:SetPoint("TOPLEFT", 5, -5)
	tq2frame.titlereg:SetPoint("TOPRIGHT", 0, 0)
	tq2frame.titlereg:SetHeight(20)
	tq2frame.titlereg:SetScript("OnMouseDown", function(frame)
		tq2frame:StartMoving()
	end)
	tq2frame.titlereg:SetScript("OnMouseUp", function(frame)
		tq2frame:StopMovingOrSizing()
		local s = tq2frame:GetEffectiveScale()
		prdb.point = "BOTTOMLEFT"
		prdb.x = tq2frame:GetLeft() * s
		prdb.y = tq2frame:GetBottom() * s
	end)

	tq2frame.title = tq2frame:CreateFontString(nil, "ARTWORK")
	tq2frame.title:SetFontObject(fontN)
	tq2frame.title:SetPoint("TOP", 0, -4)
	tq2frame.title:SetText("tomQuest2")

	if LightHeaded then
		tq2frame.scrollQL = CreateFrame("ScrollFrame", "tomQuest2ScrollFrameQL", tq2frame, "UIPanelScrollFrameTemplate")
		tq2frame.scrollQL:SetPoint("TOPLEFT", 30, -30)
		tq2frame.scrollQL:SetPoint("BOTTOMRIGHT", -((prdb.width / 2) + 35), 55)

		tq2frame.scrollQLchild = CreateFrame("Frame", "tomQuest2ScrollFrameQLChild", tq2frame.scrollQL)
		tq2frame.scrollQLchild:SetWidth(10)
		tq2frame.scrollQLchild:SetHeight(10)

		tq2frame.QLtext = CreateFrame("SimpleHTML", "tomQuest2QLHTML", tq2frame.scrollQLchild)
		tq2frame.QLtext:SetPoint("TOPLEFT", 0, 0)
		tq2frame.QLtext:SetHeight(300)

		local fontH1 = CreateFont("tq2H1Font")
		fontH1:SetFont(LSM:Fetch("font", prdb.font), prdb.fontSizeH1, "OUTLINE")
		local fontH2 = CreateFont("tq2H2Font")
		fontH2:SetFont(LSM:Fetch("font", prdb.font), prdb.fontSizeH2, "OUTLINE")
		tq2frame.QLtext:SetFontObject(fontN)
		tq2frame.QLtext:SetFontObject("h1", fontH1)
		tq2frame.QLtext:SetFontObject("h2", fontH2)

		tq2frame.scrollLH = CreateFrame("ScrollFrame", "tomQuest2ScrollFrameLH", tq2frame, "UIPanelScrollFrameTemplate")
		tq2frame.scrollLH:SetPoint("TOPLEFT", (prdb.width / 2) + 30, -30)
		tq2frame.scrollLH:SetPoint("BOTTOMRIGHT", -35, 55)

		tq2frame.scrollLHchild = CreateFrame("Frame", "tomQuest2ScrollFrameLHChild", tq2frame.scrollLH)
		tq2frame.scrollLHchild:SetWidth(10)
		tq2frame.scrollLHchild:SetHeight(10)

		tq2frame.LHtext = CreateFrame("SimpleHTML", "tomQuest2LHHTML", tq2frame.scrollLHchild)
		tq2frame.LHtext:SetPoint("TOPLEFT", 0, 0)
		tq2frame.LHtext:SetHeight(300)
		tq2frame.LHtext:SetFontObject(fontN)

		local function qlFixOnUpdate(self, elapsed)
			tq2frame.QLtext:SetText(tq2frame.QLtext.updateText)
			tq2frame.QLtext:SetScript("OnUpdate", nil)
		end

		local function lhFixOnUpdate(self, elapsed)
			tq2frame.LHtext:SetText(tq2frame.LHtext.updateText)
			tq2frame.LHtext:SetScript("OnUpdate", nil)
		end

		function tq2frame.QLtext:MySetText(msg)
			tq2frame.QLtext.updateText = msg
			tq2frame.QLtext:SetScript("OnUpdate", qlFixOnUpdate)
		end

		function tq2frame.LHtext:MySetText(msg)
			tq2frame.LHtext.updateText = msg
			tq2frame.LHtext:SetScript("OnUpdate", lhFixOnUpdate)
		end

		function tq2frame.QLtext:UpdateSize()
			tq2frame.QLtext:SetWidth((prdb.width / 2) - 65)
			tq2frame.QLtext:MySetText(tq2frame.QLtext.updateText)
			tq2frame.scrollQL:UpdateScrollChildRect()
		end

		function tq2frame.LHtext:UpdateSize()
			tq2frame.LHtext:SetWidth((prdb.width / 2) - 65)
			tq2frame.LHtext:MySetText(tq2frame.LHtext.updateText)
			tq2frame.scrollLH:UpdateScrollChildRect()
		end
		tq2frame.QLtext:EnableMouse(true)
		tq2frame.QLtext:SetHyperlinksEnabled(true)
 		tq2frame.QLtext:SetScript("OnHyperlinkEnter", function(...) module:OnHyperlinkEnter(...) end)
 		tq2frame.QLtext:SetScript("OnHyperlinkLeave", function() tq2frame.tooltip:Hide() end)
		
		tq2frame.LHtext:EnableMouse(true)
		tq2frame.LHtext:SetHyperlinksEnabled(true)
		tq2frame.LHtext:SetScript("OnHyperlinkClick", function(...) DEFAULT_CHAT_FRAME:AddMessage("youyo") end)
--~  		tq2frame.LHtext:SetScript("OnHyperlinkClick", function(...) module:OnHyperlinkClick(...) end)
 		tq2frame.LHtext:SetScript("OnHyperlinkEnter", function(...) module:OnHyperlinkEnter(...) end)
 		tq2frame.LHtext:SetScript("OnHyperlinkLeave", function() tq2frame.tooltip:Hide() end)

		tq2frame.scrollQL:SetScrollChild(tq2frame.scrollQLchild)
		tq2frame.scrollLH:SetScrollChild(tq2frame.scrollLHchild)

		tq2frame.subScrollLH = CreateFrame("Frame", "tomQuest2SubScrollLH", tq2frame)
		tq2frame.subScrollLH:SetPoint("TOPLEFT", tq2frame, "BOTTOMLEFT", prdb.width/2, 55)
		tq2frame.subScrollLH:SetPoint("BOTTOMRIGHT", tq2frame, "BOTTOMRIGHT")

		tq2frame.tooltip = CreateFrame("GameTooltip", "tq2frameTooltip", UIParent, "GameTooltipTemplate")
	
		tq2frame.icon = CreateFrame("Button", nil, tq2frame.subScrollLH)
		tq2frame.icon:SetHeight(64)
		tq2frame.icon:SetWidth(64)
		tq2frame.icon:SetNormalTexture("Interface\\AddOns\\tomQuest2\\media\\wh_icon")
		tq2frame.icon:SetHighlightTexture("Interface\\AddOns\\tomQuest2\\media\\wh_icon_hover")
		tq2frame.icon:GetHighlightTexture():SetBlendMode("BLEND")
		tq2frame.icon:SetPoint("BOTTOMRIGHT", -10, -10)

		StaticPopupDialogs["lightHeaded_ABOUT_DIALOG"] = {
			text = "lightHeaded (C) 2007-2008 by Jim Whitehead\n\nThis add-on only provides you with the comments and some basic information on quests. For the full information, be sure to visit http://www.wowhead.com",
			button1 = TEXT(OKAY),
			OnAccept = function()end,
			timeout = 0,
			hideOnEscape = 1
		}

		tq2frame.icon:SetScript("OnClick", function()
			StaticPopup_Show("lightHeaded_ABOUT_DIALOG")
		end)

		tq2frame.page = tq2frame.subScrollLH:CreateFontString(nil, "ARTWORK")
		tq2frame.page:SetFontObject(fontN)
		tq2frame.page:SetPoint("BOTTOM", 0, 20)

		tq2frame.prev = CreateFrame("Button", nil, tq2frame.subScrollLH)
		tq2frame.prev:SetWidth(32)
		tq2frame.prev:SetHeight(32)
		tq2frame.prev:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up")
		tq2frame.prev:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down")
		tq2frame.prev:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled")
		tq2frame.prev:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
		tq2frame.prev:SetPoint("RIGHT", tq2frame.page, "LEFT", -10, 0)

		module:SetFrameTooltip(tq2frame.prev, LI["Previous page"])
		tq2frame.prev:SetScript("OnClick", function(frame)
			if tq2frame.currentPage then
				module:updateQuestsLightHeadedInformations(tq2frame.uid, tq2frame.currentPage - 1)
			end
		end)

		tq2frame.next = CreateFrame("Button", nil, tq2frame.subScrollLH)
		tq2frame.next:SetWidth(32)
		tq2frame.next:SetHeight(32)
		tq2frame.next:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up")
		tq2frame.next:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down")
		tq2frame.next:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled")
		tq2frame.next:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
		tq2frame.next:SetPoint("LEFT", tq2frame.page, "RIGHT", 10, 0)

		module:SetFrameTooltip(tq2frame.next, LI["Next page"])
		tq2frame.next:SetScript("OnClick", function(frame)
			if tq2frame.currentPage then
				module:updateQuestsLightHeadedInformations(tq2frame.uid, tq2frame.currentPage + 1)
			end
		end)

		tq2frame.resize:SetScript("OnMouseDown", function(frame)
			tq2frame:StartSizing()
		end)
		tq2frame.resize:SetScript("OnMouseUp", function(frame)
			tq2frame:StopMovingOrSizing()
			local s = tq2frame:GetEffectiveScale()
			prdb.point = "BOTTOMLEFT"
			prdb.x = tq2frame:GetLeft() * s
			prdb.y = tq2frame:GetBottom() * s
			prdb.width = tq2frame:GetWidth()
			prdb.height = tq2frame:GetHeight()
			tq2frame.scrollLH:SetPoint("TOPLEFT", (prdb.width / 2) + 30, -30)
			tq2frame.scrollLH:SetPoint("BOTTOMRIGHT", -35, 55)
			tq2frame.scrollQL:SetPoint("TOPLEFT", 30, -30)
			tq2frame.scrollQL:SetPoint("BOTTOMRIGHT", -((prdb.width / 2) + 35), 55)
			tq2frame.QLtext:UpdateSize()
			tq2frame.LHtext:UpdateSize()
			tq2frame.scrollQL:UpdateScrollChildRect()
			tq2frame.scrollLH:UpdateScrollChildRect()
		end)
	else
		tq2frame.scrollQL = CreateFrame("ScrollFrame", "tomQuest2ScrollFrameQL", tq2frame, "UIPanelScrollFrameTemplate")
		tq2frame.scrollQL:SetPoint("TOPLEFT", 30, -30)
		tq2frame.scrollQL:SetPoint("BOTTOMRIGHT", -35, 55)

		tq2frame.scrollQLchild = CreateFrame("Frame", "tomQuest2ScrollFrameQLChild", tq2frame.scrollQL)
		tq2frame.scrollQLchild:SetWidth(10)
		tq2frame.scrollQLchild:SetHeight(10)

		tq2frame.QLtext = CreateFrame("SimpleHTML", "tomQuest2QLHTML", tq2frame.scrollQLchild)
		tq2frame.QLtext:SetPoint("TOPLEFT", 0, 0)
		tq2frame.QLtext:SetHeight(300)
		tq2frame.QLtext:SetFontObject(fontN)

		local function qlFixOnUpdate(self, elapsed)
			tq2frame.QLtext:SetText(tq2frame.QLtext.updateText)
			tq2frame.QLtext:SetScript("OnUpdate", nil)
		end

		function tq2frame.QLtext:MySetText(msg)
			tq2frame.QLtext.updateText = msg
			tq2frame.QLtext:SetScript("OnUpdate", qlFixOnUpdate)
		end

		function tq2frame.QLtext:UpdateSize()
			tq2frame.QLtext:SetWidth(prdb.width - 65)
			tq2frame.QLtext:MySetText(tq2frame.QLtext.updateText)
			tq2frame.scrollQL:UpdateScrollChildRect()
		end

		tq2frame.scrollQL:SetScrollChild(tq2frame.scrollQLchild)

		tq2frame.tooltip = CreateFrame("GameTooltip", "tq2frameTooltip", UIParent, "GameTooltipTemplate")

		tq2frame.resize:SetScript("OnMouseDown", function(frame)
			tq2frame:StartSizing()
		end)
		tq2frame.resize:SetScript("OnMouseUp", function(frame)
			tq2frame:StopMovingOrSizing()
			local s = tq2frame:GetEffectiveScale()
			prdb.point = "BOTTOMLEFT"
			prdb.x = tq2frame:GetLeft() * s
			prdb.y = tq2frame:GetBottom() * s
			prdb.width = tq2frame:GetWidth()
			prdb.height = tq2frame:GetHeight()
			tq2frame.scrollQL:SetPoint("TOPLEFT", 30, -30)
			tq2frame.scrollQL:SetPoint("BOTTOMRIGHT", -35, 55)
			tq2frame.QLtext:UpdateSize()
			tq2frame.scrollQL:UpdateScrollChildRect()
		end)
	end
	tq2frame:SetAlpha(1)
	tq2frame:Show()
end

function module:displayQuestInformations(event, uid)
	if not core.quests[uid] then return end
	if not tomQuest2Frame then module:CreateGUI() end
	local tq2frame = tomQuest2Frame
	if tq2frame:IsVisible() and tq2frame.uid and tq2frame.uid == uid then
		tq2frame:Hide()
	else
		tq2frame:SetPoint(prdb.point, prdb.x, prdb.y)
		tq2frame.uid = uid
		local qlText
		if not core.quests[uid].description then
			qlText = format("<HTML><BODY><H1>%s</H1><BR/><P>%s</P></BODY></HTML>", core.quests[uid].title, LI["Quests Informations not yet available, please try once synchronization with party is complete"])
		else
			local rewards = ""
			if (core.quests[uid].titleReward ~= "" or core.quests[uid].rewardSpellName ~= "" or (core.quests[uid].numRewards + core.quests[uid].numChoices + core.quests[uid].money + core.quests[uid].honor + core.quests[uid].talent) > 0 ) then
				local money, honor, talent, title, spell, itemReward, itemChoice = "", "", "", "", "", "" ,""
				if core.quests[uid].money > 0 then
					local gold = floor(math.abs(core.quests[uid].money) / 10000)
					local silver = mod(floor(math.abs(core.quests[uid].money) / 100), 100)
					local copper = mod(floor(math.abs(core.quests[uid].money)), 100)
					if gold > 0 then
						money = format("%s |cff%s%d|r.|cff%s%d|r.|cff%s%d|r<BR/>", LI["You will rececive:"], COLOR_GOLD, gold, COLOR_SILVER, silver, COLOR_COPPER, copper)
					elseif silver > 0 then
						money = format("%s |cff%s%d|r.|cff%s%d|r<BR/>", LI["You will rececive:"], COLOR_SILVER, silver, COLOR_COPPER, copper)
					else
						money = format("%s |cff%s%d|r<BR/>", LI["You will rececive:"], COLOR_COPPER, copper)
					end
				end
				if core.quests[uid].honor > 0 then
					honor = format(LI["Honor points: %s<BR/>"], core.quests[uid].honor)
				end
				if core.quests[uid].talent > 0 then
					talent = format(LI["Talent points: %d<BR/>"], core.quests[uid].talent)
				end
				if core.quests[uid].titleReward ~= "" then
					title = format(LI["Title: %s<BR/>"], core.quests[uid].titleReward)
				end
				if core.quests[uid].rewardSpellName ~= "" then
					spell = string.format("|T%s:32|t %s", core.quests[uid].rewardSpellTexture, core.quests[uid].rewardSpellName)				
				end
				if core.quests[uid].numRewards > 0 then
					for i = 1, core.quests[uid].numRewards do
						local numItems = core.quests[uid].itemRewards[i].numItems == 1 and "" or string.format(" %d",core.quests[uid].itemRewards[i].numItems)
						itemReward = string.format("%s%s|T%s:32|t%s<BR/>", itemReward, numItems, core.quests[uid].itemRewards[i].texture, core.quests[uid].itemRewards[i].link)
					end
				end
				if core.quests[uid].numChoices > 0 then
					for i = 1, core.quests[uid].numChoices do
						local numItems = core.quests[uid].itemChoices[i].numItems == 1 and "" or string.format(" %d", core.quests[uid].itemChoices[i].numItems)
						itemChoice = string.format("%s%s|T%s:32|t%s<BR/>", itemChoice, numItems, core.quests[uid].itemChoices[i].texture, core.quests[uid].itemChoices[i].link)
					end
					itemChoice = string.format(LI["Choose one of the following reward:<BR/>%s"], itemChoice)
				end
				rewards = format("<BR/><H2>%s</H2><P> </P><P>%s%s%s%s%s%s%s</P>", LI["Rewards:"], money, honor, talent, title, spell, itemReward, itemChoice)
			end
			qlText = format("<HTML><BODY><H1>%s</H1><BR/><P>%s</P><BR/><H2>%s</H2><BR/><P>%s</P>%s</BODY></HTML>", core.quests[uid].title, string.gsub(string.gsub(core.quests[uid].objective, "<", "&lt;"), ">", "&gt;"), LI["Description:"], string.gsub(string.gsub(core.quests[uid].description, "<", "&lt;"), ">", "&gt;"), rewards)
		end
		tq2frame.QLtext:MySetText(qlText)
		tq2frame.QLtext:UpdateSize()
		if LightHeaded then
			local num = LightHeaded:GetNumQuestComments(uid) or 0
			local hlText = module:getQuestLightHeadedInformations(uid, 1)
			tq2frame.currentPage = 1
			if hlText then
				tq2frame.page:SetText(format(LI["Page %s of %s"], 1, num + 1))
				tq2frame.LHtext:MySetText(hlText)
				tq2frame.LHtext:UpdateSize()
				if num == 0 then
					tq2frame.prev:Hide()
					tq2frame.next:Hide()
				else
					tq2frame.page:Show()
					tq2frame.prev:Show()
					tq2frame.next:Show()
					tq2frame.prev:Disable()
					if page == num + 1 then
						tq2frame.next:Disable()
					else
						tq2frame.next:Enable()
					end
				end
			end
		end
		tq2frame:Show()
	end
end

function module:getQuestLightHeadedInformations(uid, page)
	local text = ""
	local title = core.quests[uid].title or LightHeaded:GetQuestName(uid)

	if page == 1 then
		for uid, sharable, level, reqlev, stype, sname, sid, etype, ename, eid, exp, rep, series in LightHeaded:IterateQuestInfo(uid) do
			if reqlev ~= "" then
				reqlev = format(LI["%s|c%sRequired Level:|r %d"], "\n", prdb.color, reqlev)
			end
			if stype ~= "" and sname ~= "" then
				if stype == "npc" then
					sname = format(LI["%s|c%sStarts:|r |cFF0066FF|Hlhref:npc:%s:%s|h[%s]|h|r"], "\n", prdb.color, sid, sname, sname)
				else
					sname = format(LI["%s|c%sStarts:|r |cFFFFFFFF%s|r"], "\n", prdb.color, sname)
				end
			end
			if etype ~= "" and ename ~= "" then
				if etype == "npc" then
					ename = format(LI["%s|c%sEnds:|r |cFF0066FF|Hlhref:npc:%s:%s|h[%s]|h|r"], "\n", prdb.color, eid, ename, ename)
				else
					ename = format(LI["%s|c%sEnds:|r |cFFFFFFFF%s|r"], "\n", prdb.color, ename)
				end
				
			end
			if exp ~= "" then
				exp = format(LI["%s|c%sExperience:|r %d"], "\n", prdb.color, exp)
			end
			local repText = ""
			if rep ~= "" then
				repText = format(LI["%s|c%sReputation Gains:|r"], "\n", prdb.color)
				for name, value in rep:gmatch("([^\029]+)\029([^\029]+)") do
					repText = format("%s\n|c%s - %s |r: %s", repText, prdb.color, name, value)
				end
			end
			local seriesText = ""
			if series ~= "" then
				seriesText = format(LI["%s|c%sQuest Series:|r"], "\n", prdb.color)
				for step, id in series:gmatch("([^\029]+)\029([^\029]+)") do
					if uid == id then
						seriesText = format("%s\n|c%s%s - %s |r", seriesText, prdb.color, step, title)
					else
						seriesText = format("%s\n|c%s%s - |r|cFF0066FF|Hlhref:quest:%s|h[%s]|h|r", seriesText, prdb.color, step, id, LightHeaded:GetQuestName(id))
					end
				end
			end
			text = format(LI["|c%sName:|r %s%s|c%sQuest ID:|r |Hlhref:external:http://www.wowhead.com/?quest=%d|h%d|h%s|c%sSharable:|r %s%s|c%sLevel:|r %d%s%s%s%s%s%s"], prdb.color, title, "\n", prdb.color, uid, uid, "\n", prdb.color, sharable and LI["Yes"] or LI["No"], "\n", prdb.color, level, reqlev, sname, ename, exp, repText, seriesText)
		end
	else
		text = format(LI["|c%sComments for:|r %s%s%s"], prdb.color, title, "\n", LightHeaded:GetCommentText(uid, page - 1))
	end
	return text
end

function module:updateQuestsLightHeadedInformations(uid, page)
	local tq2frame = tomQuest2Frame
	if not tq2frame:IsVisible() then return end

	local hlText
	if #stack > 0 then
		hlText = format("%s%s", "|cFF0066FF|Hlhref:back|h[Back]|h|r\n",module:getQuestLightHeadedInformations(uid, page))
	else
		hlText = module:getQuestLightHeadedInformations(uid, page)
	end	
	tq2frame.LHtext:MySetText(hlText)
	tq2frame.LHtext:UpdateSize()

	local num = LightHeaded:GetNumQuestComments(uid)
	if not num then return end
	tq2frame.page:SetText(format(LI["Page %s of %s"], page, num + 1))

	tq2frame.page:Show()
	tq2frame.prev:Show()
	tq2frame.next:Show()

	if page == 1 then
		tq2frame.prev:Disable()
	else
		tq2frame.prev:Enable()
	end
	if page == num + 1 then
		tq2frame.next:Disable()
	else
		tq2frame.next:Enable()
	end

	tq2frame.uid = uid
	tq2frame.currentPage = page
	tq2frame.scrollLH:UpdateScrollChildRect()
end

function module:OnHyperlinkClick(frame, link)
	local tq2frame = tomQuest2Frame
	if not link then return end
	if link:match("^lhref:npc") then
		table.insert(stack, tq2frame.currentPage)
		table.insert(stack, tq2frame.uid)
		local id, name = link:match("^lhref:npc:(.+):(.+)$")
		id = tonumber(id)
		local text = format("|cFF0066FF|Hlhref:back|h[Back]|h|r\n\n\n%s|cFFFFFFFF%s|r:\n", LI["Known locations for "], name)
		if LightHeaded:LoadNPCData(id) then
			for c,z,x,y in LightHeaded:IterateNPCLocs(id) do
				text = format("%s\n|cFF0066FF|Hlhref:zcoord:%s:%s:%s:%s:%s|h[%s]|h|r", text, c, z, x, y, name, LightHeaded:GetCoordText(x, y))
			end
		else
			text = format("%s\n%s", text, LI["Location unknown"])
		end
		tq2frame.LHtext:MySetText(text)
		tq2frame.LHtext:UpdateSize()
		tq2frame.scrollLH:UpdateScrollChildRect()
		tq2frame.next:Disable()
		tq2frame.prev:Disable()
	elseif link:match("^lhref:coord") then
		local xt, yt, uid = link:match("^lhref:coord:([^:]+):([^:]+):(.+)")
		local x = tonumber(xt)
		local y = tonumber(yt)
		if not x or not y then
			self:Print(LI["The coordinate you clicked on was not valid: %s"], link)
			return
		end
		local note
		if not tonumber(uid) then
			note = uid
		else
			note = string.format("[LH] %s", LightHeaded:GetQuestName(uid))
		end
		if TomTom then
			TomTom:AddWaypoint(x, y, note)
		elseif Cartographer_Waypoints and type(Cartographer_Waypoints.AddLHWaypoint) == "function" then
			Cartographer_Waypoints:AddLHWaypoint(nil, nil, x, y, note)
		elseif MN_ThottInterface and type(MN_ThottInterface_Local) == "function" then
			MN_ThottInterface_Local(x, y, note)
		elseif Cartographer_Waypoints then
			self:Print(LI["LightHeaded waypoints are not supported in this version of Cartographer_Waypoints."])
		end
	elseif link:match("^lhref:zcoord") then
		local c, z, x, y, uid = link:match("^lhref:zcoord:([^:]+):([^:]+):([^:]+):([^:]+):(.+)")
		c, z, x, y = tonumber(c), tonumber(z), tonumber(x), tonumber(y)

		local note
		if not tonumber(uid) then
			note = uid
		else
			note = string.format("[LH] %s", LightHeaded:GetQuestName(uid))
		end

		if TomTom and type(TomTom.AddZWaypoint) == "function" then
			TomTom:AddZWaypoint(c, z, x, y, note)
		elseif  Cartographer_Waypoints and type(Cartographer_Waypoints.AddLHWaypoint) == "function" then
			Cartographer_Waypoints:AddLHWaypoint(c, z, x, y, note)
		elseif MN_ThottInterface and type(MN_ThottInterface_Legacy) == "function" then
			MN_ThottInterface_Legacy(c, z, x, y, note)
		elseif TomTom then
			self:Print(LI["NPC locations are not supported in this version of TomTom."])
		elseif Cartographer_Waypoints then
			self:Print(LI["LightHeaded waypoints are not supported in this version of Cartographer_Waypoints."])
		end
	elseif link:match("^lhref:external") then
		local url = link:match("^lhref:external:(.+)$")
		ChatFrameEditBox:Show()
		ChatFrameEditBox:SetText(url)
		ChatFrameEditBox:SetFocus()

	elseif link:match("^lhref:quest:") then
		table.insert(stack, tq2frame.currentPage)
		table.insert(stack, tq2frame.uid)

		local uid = link:match("^lhref:quest:(.+)$")
		uid = tonumber(uid)

		module:updateQuestsLightHeadedInformations(uid, 1)
	elseif link:match("^lhref:back$") then
		local uid = table.remove(stack)
		local page = table.remove(stack)

		module:updateQuestsLightHeadedInformations(uid, page)
	elseif link:match("^lhref:item:") then
		link = link:sub(7)
		local name, chatLink = GetItemInfo(link)
		if name then
			if IsControlKeyDown() then
				DressUpItemLink(link)
			elseif IsShiftKeyDown() and ChatFrameEditBox:IsVisible() then
				ChatFrameEditBox:Insert(chatLink)
			else
				SetItemRef(link)
			end
		elseif IsAltKeyDown() then
			local tooltip = tq2frame.tooltip
			tooltip.linkForced = link
			tooltip:SetOwner(tq2frame.text, "ANCHOR_CURSOR")
			tooltip:SetHyperlink(link)
		end
	else
		self:Print(LI["Unmanaged lhref:%s "], link)
	end
end

function module:OnHyperlinkEnter(frame, link)
	local tq2frame = tomQuest2Frame
	if not link then return end
	if link:match("^lhref:item") then
		link = link:sub(7)
		tq2frame.tooltip:SetOwner(frame,"ANCHOR_CURSOR")
		tq2frame.tooltip:SetHyperlink(link)
		tq2frame.tooltip:Show()
	elseif link:match("^item") then
		tq2frame.tooltip:SetOwner(frame,"ANCHOR_CURSOR")
		tq2frame.tooltip:SetHyperlink(link)
		tq2frame.tooltip:Show()
	end
end
