local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("questsTracker", "AceEvent-3.0", "AceHook-3.0")
local LQTr = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-QuestsTracker")
local LSM = LibStub("LibSharedMedia-3.0")
local LibQTip = LibStub("LibQTip-1.0")


--------------------------------------
--- Copy global functions to local ---
--------------------------------------
local pairs = pairs
local format = string.format
local ToggleFrame = ToggleFrame
local SetAbandonQuest = SetAbandonQuest
local SelectQuestLogEntry = SelectQuestLogEntry
local QuestLogPushQuest = QuestLogPushQuest
local MouseIsOver = MouseIsOver
local ItemRefTooltip = ItemRefTooltip
local GetQuestLogSelection = GetQuestLogSelection
local GetQuestLogPushable = GetQuestLogPushable
local GetQuestLink = GetQuestLink
local CreateFrame = CreateFrame
local CreateFont = CreateFont
local UnitIsDead = UnitIsDead
local unpack = unpack

local defaults = {
	profile = {
		displayHeader = false,
		questscellpadding = 5,
		collapseButtonSize = 24,
		colorizeHeaderName = true,
		colorizeHeaderLevel = true,
		hideTrackerInCombat = true,
		hideCompletedQuest = 3,
		hideCompletedObjective = 3,
		onObjectiveProgress = true,
		onSubZone = true,
		point = "CENTER",
		x = 0,
		y = 0,
		showAnchor = true,
		font = "Friz Quadrata TT",
		headerFontSize = 12,
		normalFontSize = 10,
		backDropColor = {0.09, 0.09, 0.09, 1},
		borderColor = {1, 1, 1, 1},
		scale = 1,
		maxWidth = 300,
		objectivescellpadding = 10,
		chatlink = true,
		questsobjectives = true,
		clicktooltip = true,
		sharequest = true,
		trackquest = true,
		abandonquest = true,
		questlog = true,
		overtooltip = true,
		colorizeQuestName = true,
		colorizeQuestLevel = true,
		colorizeQuestObjective = true,
		anchorPosition = 2,
	},
}

local prdb, chdb
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("questsTracker", defaults, "Default")
	prdb = self.db.profile
	chdb = self.db.char
	chdb.questsTracked = chdb.questsTracked or {}
	chdb.questsAutoTracked = chdb.questsAutoTracked or {}
	chdb.questsTrackedZone = {}
	chdb.questsTrackedByZone = {}
	chdb.zoneDisplay = chdb.zoneDisplay or {}
end
local subZone = ""
module.modName = LQTr["Quests Tracker"]

local numTrackedQuests = 0
local tracker, trackerAnchor
local anchor = CreateFrame("Button", nil, UIParent)
anchor:SetHeight(24)
anchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5}, tile = true, tileSize = 16})
anchor:SetBackdropColor(0.09, 0.09, 0.19, 0.5)
anchor:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
local text = anchor:CreateFontString(nil, nil, "GameFontNormalSmall")
text:SetPoint("CENTER")
text:SetText(LQTr["Tracker"])
anchor:SetWidth(text:GetStringWidth() + 8)
anchor:SetMovable(true)
anchor:RegisterForDrag("LeftButton")
anchor:SetClampedToScreen(true)
anchor:SetScript("OnDragStart", function(self)
	self:StartMoving()
end)
anchor:SetScript("OnDragStop", function(self)
	self:StopMovingOrSizing()
	prdb.point, prdb.x, prdb.y = "BOTTOMLEFT", self:GetLeft(), self:GetBottom()
	trackerAnchor = anchor:GetCenter() > UIParent:GetWidth()*0.5 and "ANCHOR_LEFT" or "ANCHOR_RIGHT"
	LibQTip:Release(tracker)
	module:updateQuestTracker()
end)


local options
function module:getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = function(info) 
				if type(prdb[info[#info]]) == "table" then
					return unpack(prdb[info[#info]])
				else
					return prdb[info[#info]]
				end
			end,
			set = function(info, arg1, arg2, arg3, arg4)
				if arg2 then
					prdb[info[#info]][1] = arg1
					prdb[info[#info]][2] = arg2
					prdb[info[#info]][3] = arg3
					prdb[info[#info]][4] = arg4
				else
					prdb[info[#info]] = arg1
				end
				if info[#info] == "showAnchor" then
					if prdb.showAnchor then
						anchor:Show()
					else
						anchor:Hide()
					end
				elseif info[#info] == "hideTrackerInCombat" then
					if prdb.hideTrackerInCombat then
						module:RegisterEvent("PLAYER_REGEN_ENABLED","updateQuestsTracker")
						module:RegisterEvent("PLAYER_REGEN_DISABLED","updateQuestsTracker")
					else
						module:UnregisterEvent("PLAYER_REGEN_ENABLED")
						module:UnregisterEvent("PLAYER_REGEN_DISABLED")
					end
				elseif info[#info] == "hideCompletedQuest" then
					module:initTrackedQuest()
				end
				LibQTip:Release(tracker)
				module:updateQuestTracker()
			end,
			args = {
				onObjectiveProgress = {
					name = LQTr["Auto track quests on objective progress"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				onSubZone = {
					name = LQTr["Auto track quests on sub zone change"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				showAnchor = {
					name = LQTr["Show quest tracker anchor"],
					type = "toggle",
					width = "full",
					order = 8,
				},
				anchorPosition = {
					name = LQTr["Anchor position"],
					type = "select",
					values = {[1] = "TOPLEFT", [2] = "TOP", [3] = "TOPRIGHT", [4] = "BOTTOMLEFT", [5] = "BOTTOM", [6] = "BOTTOMRIGHT"},
					width = "full",
					order = 9,
				},
				trackerheader = {
					type = "header",
					name = LQTr["Tracker Settings"],
					order = 10,
				},
				hideTrackerInCombat = {
					name = LQTr["Hide quest tracker in combat"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				hideCompletedQuest = {
					name = LQTr["Hide completed quest"],
					type = "select",
					values = {[1] = LQTr["When you've completed the quest"], [2] = LQTr["When everyone in the party have completed the quest"], [3] = LQTr["Never"]},
					width = "full",
					order = 12,
				},
				hideCompletedObjective = {
					name = LQTr["Hide completed quests Objectives"],
					type = "select",
					values = {[1] = LQTr["When you've completed the objective"], [2] = LQTr["When everyone in the party have completed the objective"], [3] = LQTr["Never"]},
					width = "full",
					order = 13,
				},
				font = {
					name = LQTr["Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					width = "full",
					order = 14,
				},
				headerFontSize = {
					name = LQTr["Quests font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order=15,
				},
				normalFontSize = { 
					name = LQTr["Quests Objectives font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order=16,
				},
				maxWidth = {
					name = LQTr["Quests Tracker column max width"],
					type="range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order=17,
				},
				objectivescellpadding = {
					name = LQTr["Quests objectives cell padding"],
					type="range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order=18,
				},
				scale = {
					name = LQTr["Quests Tracker scale"],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order=19,
				},
				backDropColor = {
					name = LQTr["Quests Tracker background color"],
					type = "color",
					hasAlpha = true,
					width = "full",
					order = 20,
				},
				borderColor = {
					name = LQTr["Quests Tracker border color"],
					type = "color",
					hasAlpha = true,
					width = "full",
					order = 21,
				},
				colorizeQuestName = {
					name = LQTr["Colorize Quests Name"],
					type = "toggle",
					width = "full",
					order = 22,
				},
				colorizeQuestLevel = {
					name = LQTr["Colorize Quests Level"],
					type = "toggle",
					width = "full",
					order = 23,
				},
				colorizeQuestObjective = {
					name = LQTr["Colorize Quests Objectives"],
					type = "toggle",
					width = "full",
					order = 24,
				},
				dispayHeader = {
					name = LQTr["Dispay quests by zone"],
					type = "toggle",
					width = "full",
					order = 25,
				},
				questscellpadding = {
					name = LQTr["Quests cell padding"],
					type="range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 26,
				},
				collapseButtonSize = {
					name = LQTr["Collapse Button Size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 27,
				},
				colorizeHeaderName = {
					name = LQTr["Colorize Quests Header Name"],
					type = "toggle",
					width = "full",
					order = 28,
				},
				colorizeHeaderLevel = {
					name = LQTr["Colorize Quests Header Level"],
					type = "toggle",
					width = "full",
					order = 29,
				},
				trackerinteraction = {
					type = "header",
					name = LQTr["Tracker interaction"],
					order = 30,
				},
				chatlink  = {
					name = LQTr["Toggle chatlink when shift - right clicking on a quest"],
					type = "toggle",
					width = "full",
					order = 31,
				},
				questsobjectives = {
					name = LQTr["Toggle insert of quest objectives in chat when ctrl - right clicking on a quest"],
					type = "toggle",
					width = "full",
					order = 32,
				},
				clicktooltip = {
					name = LQTr["Toggle display of quest tooltip when right clicking on a quest"],
					type = "toggle",
					width = "full",
					order = 33,
				},
				sharequest = {
					name = LQTr["Toggle quest sharing when ctrl - left clicking on a quest"],
					type = "toggle",
					width = "full",
					order = 34,
				},
				trackquest = {
					name = LQTr["Toggle quest tracking when shift - left clicking on a quest"],
					type = "toggle",
					width = "full",
					order = 35,
				},
				abandonquest = {
					name = LQTr["Toggle quest abandon when alt - left clicking on a quest"],
					type = "toggle",
					width = "full",
					order = 36,
				},
				questlog = {
					name = LQTr["Toggle questlog display when left clicking on a quest"],
					type = "toggle",
					width = "full",
					order = 37,
				},
				overtooltip = {
					name = LQTr["Toggle quest tooltip when mouse over a quest"],
					type = "toggle",
					width = "full",
					order = 38,
				},
			},
		}
	end
	return options
end

function module:OnEnable()
	MAX_WATCHABLE_QUESTS = MAX_QUESTS
	anchor:SetPoint(prdb.point, prdb.x, prdb.y)
	if not prdb.showAnchor then anchor:Hide() else anchor:Show() end
	trackerAnchor = anchor:GetCenter() > UIParent:GetWidth()*0.5 and "ANCHOR_LEFT" or "ANCHOR_RIGHT"
	if prdb.hideTrackerInCombat then
		self:RegisterEvent("PLAYER_REGEN_ENABLED","updateQuestsTracker")
		self:RegisterEvent("PLAYER_REGEN_DISABLED","updateQuestsTracker")
	end
	self:RegisterEvent("MINIMAP_ZONE_CHANGED", "zoneChanged")
	self:RegisterEvent("ZONE_CHANGED", "zoneChanged")
	self:RegisterEvent("ZONE_CHANGED_INDOORS", "zoneChanged")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "zoneChanged")
	self:SecureHook("AddQuestWatch")
	self:SecureHook("RemoveQuestWatch")
	self:SecureHook("QuestWatch_Update")
	core.RegisterCallback(self, "initTrackedQuest")
	core.RegisterCallback(self, "endPlayerQuest")
	core.RegisterCallback(self, "endPartyQuest")
	core.RegisterCallback(self, "playerQuestComplete", "playerQuestChanged")
	core.RegisterCallback(self, "playerQuestFailed", "playerQuestChanged")
	core.RegisterCallback(self, "partyQuestComplete", "partyQuestChanged")
	core.RegisterCallback(self, "partyQuestFailed", "partyQuestChanged")
	core.RegisterCallback(self, "playerQuestObjectiveProgress")
	core.RegisterCallback(self, "partyQuestObjectiveProgress")
	core.RegisterCallback(self, "playerQuestLost")
	core.RegisterCallback(self, "partyQuestLost")	
	core.RegisterCallback(self, "toggleTrackedQuest")
	core.RegisterCallback(self, "questObjectiveAdded")
	core.RegisterCallback(self, "questDescriptionAdded")
	core.RegisterCallback(self, "updateQuestsTracker")
end

function module:OnDisable()
	anchor:Hide()
	LibQTip:Release(tracker)
	tracker = nil
	core.UnregisterAllCallbacks(self)
end
--------------
--- Events ---
--------------
function module:zoneChanged()
	if UnitIsDead("player") then return end
	local trackerNeedUpdate = false
	subZone = GetSubZoneText() or ""
	for uid in pairs(chdb.questsAutoTracked) do
		if uid and core.quests[uid] and core.quests[uid].id and core.quests[uid].numObjectives then
			RemoveQuestWatch(core.quests[uid].id)
		elseif uid and core.quests[uid] then
			chdb.questsAutoTracked[uid] = nil
			chdb.questsTracked[uid] = nil
			numTrackedQuests = numTrackedQuests - 1
			trackerNeedUpdate = true
			if chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] then
				chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] = nil
				chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = chdb.questsTrackedByZone[core.quests[uid].zone].numQuests - 1
				if chdb.questsTrackedByZone[core.quests[uid].zone].numQuests == 0 then
					chdb.questsTrackedByZone[core.quests[uid].zone].quests = nil
					chdb.questsTrackedByZone[core.quests[uid].zone] = nil
					chdb.questsTrackedZone[core.quests[uid].zone] = nil
				end
			end
		end
	end
	if prdb.onSubZone and subZone ~= "" then
		for uid in pairs(core.quests) do
			if not chdb.questsTracked[uid] and ((core.quests[uid].objective and core.quests[uid].objective:find(subZone, 1, true)) or (core.quests[uid].description and core.quests[uid].description:find(subZone, 1, true))) then
				chdb.questsAutoTracked[uid] = true
				if core.quests[uid].id and core.quests[uid].numObjectives > 0 then
					AddQuestWatch(core.quests[uid].id)
				else
					chdb.questsTracked[uid] = true
					numTrackedQuests = numTrackedQuests + 1
					trackerNeedUpdate = true
					if prdb.hideCompletedQuest == 3 or (prdb.hideCompletedQuest == 2 and not core.quests[uid].partyComplete) or (prdb.hideCompletedQuest == 1 and not core.quests[uid].playerComplete) then
						if not chdb.questsTrackedZone[core.quests[uid].zone] then
							chdb.questsTrackedZone[core.quests[uid].zone] = true
							chdb.questsTrackedByZone[core.quests[uid].zone] = {}
							chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = 0
							chdb.questsTrackedByZone[core.quests[uid].zone].quests = {}
						end
						chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] = true
						chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = chdb.questsTrackedByZone[core.quests[uid].zone].numQuests + 1
					end
				end
			end
		end
	end
	if trackerNeedUpdate then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end
-------------
--- Hooks ---
-------------
function module:AddQuestWatch(id)
	local uid = core.questsIdToUid[id]
	if uid and not chdb.questsTracked[uid] then
		chdb.questsTracked[uid] = true
		if prdb.hideCompletedQuest == 3 or (prdb.hideCompletedQuest == 2 and not core.quests[uid].partyComplete) or (prdb.hideCompletedQuest == 1 and not core.quests[uid].playerComplete) then
			if not chdb.questsTrackedZone[core.quests[uid].zone] then
				chdb.questsTrackedZone[core.quests[uid].zone] = true
				chdb.questsTrackedByZone[core.quests[uid].zone] = {}
				chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = 0
				chdb.questsTrackedByZone[core.quests[uid].zone].quests = {}
			end
			chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] = true
			chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = chdb.questsTrackedByZone[core.quests[uid].zone].numQuests + 1
		end
		numTrackedQuests = numTrackedQuests + 1
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	elseif uid and chdb.questsTracked[uid] then
		if prdb.hideCompletedQuest == 3 or (prdb.hideCompletedQuest == 2 and not core.quests[uid].partyComplete) or (prdb.hideCompletedQuest == 1 and not core.quests[uid].playerComplete) then
			if not chdb.questsTrackedZone[core.quests[uid].zone] then
				chdb.questsTrackedZone[core.quests[uid].zone] = true
				chdb.questsTrackedByZone[core.quests[uid].zone] = {}
				chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = 0
				chdb.questsTrackedByZone[core.quests[uid].zone].quests = {}
			end
			if not chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] then
				chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] = true
				chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = chdb.questsTrackedByZone[core.quests[uid].zone].numQuests + 1
			end
			numTrackedQuests = numTrackedQuests + 1
			LibQTip:Release(tracker)
			module:updateQuestTracker()
		end
	end
end

function module:RemoveQuestWatch(id)
	local uid = core.questsIdToUid[id]
	if uid and chdb.questsTracked[uid] then
		chdb.questsTracked[uid] = nil
		chdb.questsAutoTracked[uid] = nil
		if chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] then
			chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] = nil
			chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = chdb.questsTrackedByZone[core.quests[uid].zone].numQuests - 1
			if chdb.questsTrackedByZone[core.quests[uid].zone].numQuests == 0 then
				chdb.questsTrackedByZone[core.quests[uid].zone].quests = nil
				chdb.questsTrackedByZone[core.quests[uid].zone] = nil
				chdb.questsTrackedZone[core.quests[uid].zone] = nil
			end
		end
		numTrackedQuests = numTrackedQuests - 1
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function GetNumQuestWatches()
	return numTrackedQuests
end

function module:QuestWatch_Update()
 	QuestWatchFrame:Hide()
end
-----------------
--- Callbacks ---
-----------------
function module:initTrackedQuest(event)
	numTrackedQuests = 0
	for uid in pairs(chdb.questsTracked) do
		if not core.quests[uid] then
			chdb.questsTracked[uid] = nil
			chdb.questsAutoTracked[uid] = nil
			for zone in pairs (chdb.questsTrackedByZone) do
				if chdb.questsTrackedByZone[zone].quests[uid] then
					chdb.questsTrackedByZone[zone].quests[uid] = nil
					chdb.questsTrackedByZone[zone].numQuests = chdb.questsTrackedByZone[zone].numQuests - 1
					if chdb.questsTrackedByZone[zone].numQuests == 0 then
						chdb.questsTrackedByZone[zone].quests = nil
						chdb.questsTrackedByZone[zone] = nil
						chdb.questsTrackedZone[zone] = nil
					end
				end
			end
		else
			module:trackQuest(uid)
		end
	end
	LibQTip:Release(tracker)
	module:updateQuestTracker()
end

function module:endPlayerQuest(event, first, commHandler, uid, started)
	local questAddedToTracker = 0
	if prdb.onObjectiveProgress and started == 1 then
		if not chdb.questsTracked[uid] then
			questAddedToTracker = 1
			module:trackQuest(uid)
		elseif chdb.questsAutoTracked[uid] then
			chdb.questsAutoTracked[uid] = nil
		end
	end
	if not chdb.questsTracked[uid] and prdb.onSubZone and subZone ~= "" then
		if core.quests[uid].objective:find(subZone, 1, true) or core.quests[uid].description:find(subZone, 1, true) then
			chdb.questsAutoTracked[uid] = true
			questAddedToTracker = 1
			module:trackQuest(uid)
		end
	end
	if questAddedToTracker == 0 and chdb.questsTracked[uid] then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:endPartyQuest(event, uid, started)
	if chdb.questsTracked[uid] then
		if prdb.onObjectiveProgress and started == 1 and chdb.questsAutoTracked[uid] then
			chdb.questsAutoTracked[uid] = nil
		end
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	elseif prdb.onObjectiveProgress and started == 1 then
		module:trackQuest(uid)
	end
end

function module:playerQuestChanged(event, first, commHandler, uid)
	if chdb.questsTracked[uid] then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:partyQuestChanged(event, sender, uid)
	if chdb.questsTracked[uid] then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:playerQuestObjectiveProgress(event, first, commHandler, uid, objective, got, nGot)
	if prdb.onObjectiveProgress then
		chdb.questsAutoTracked[uid] = nil
		if not chdb.questsTracked[uid] then
			module:trackQuest(uid)
		else
			LibQTip:Release(tracker)
			module:updateQuestTracker()
		end
	elseif chdb.questsTracked[uid] then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:partyQuestObjectiveProgress(event, sender, uid, objective, got, nGot)
	if prdb.onObjectiveProgress then
		chdb.questsAutoTracked[uid] = nil
		if not chdb.questsTracked[uid] then
			module:trackQuest(uid)
		else
			LibQTip:Release(tracker)
			module:updateQuestTracker()
		end
	elseif chdb.questsTracked[uid] then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:playerQuestLost(event, first, commHandler, uid)
	if not core.quests[uid] and chdb.questsTracked[uid] then
		chdb.questsTracked[uid] = nil
		chdb.questsAutoTracked[uid] = nil
		numTrackedQuests = numTrackedQuests - 1
		for zone in pairs (chdb.questsTrackedByZone) do
			if chdb.questsTrackedByZone[zone].quests[uid] then
				chdb.questsTrackedByZone[zone].quests[uid] = nil
				chdb.questsTrackedByZone[zone].numQuests = chdb.questsTrackedByZone[zone].numQuests - 1
				if chdb.questsTrackedByZone[zone].numQuests == 0 then
					chdb.questsTrackedByZone[zone].quests = nil
					chdb.questsTrackedByZone[zone] = nil
					chdb.questsTrackedZone[zone] = nil
				end
			end
		end
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	elseif core.quests[uid] and chdb.questsTracked[uid] then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:partyQuestLost(event, uid)
	if not core.quests[uid] and chdb.questsTracked[uid] then
		chdb.questsTracked[uid] = nil
		chdb.questsAutoTracked[uid] = nil
		numTrackedQuests = numTrackedQuests - 1
		for zone in pairs (chdb.questsTrackedByZone) do
			if chdb.questsTrackedByZone[zone].quests[uid] then
				chdb.questsTrackedByZone[zone].quests[uid] = nil
				chdb.questsTrackedByZone[zone].numQuests = chdb.questsTrackedByZone[zone].numQuests - 1
				if chdb.questsTrackedByZone[zone].numQuests == 0 then
					chdb.questsTrackedByZone[zone].quests = nil
					chdb.questsTrackedByZone[zone] = nil
					chdb.questsTrackedZone[zone] = nil
				end
			end
		end
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	elseif core.quests[uid] and chdb.questsTracked[uid] then
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:toggleTrackedQuest(event, uid)
	if chdb.questsTracked[uid] then
		module:unTrackQuest(uid)
	elseif not chdb.questsTracked[uid] then
		module:trackQuest(uid)
	end
end

function module:questObjectiveAdded(event, uid)
	if prdb.onSubZone and subZone ~= "" then
		if not chdb.questsTracked[uid] and core.quests[uid].objective and core.quests[uid].objective:find(subZone, 1, true) then
			chdb.questsAutoTracked[uid] = true
			module:trackQuest(uid)
		end
	end
end

function module:questDescriptionAdded(event, uid)
	if prdb.onSubZone and subZone ~= "" then
		if not chdb.questsTracked[uid] and core.quests[uid].description	and core.quests[uid].description:find(subZone, 1, true) then
			chdb.questsAutoTracked[uid] = true
			module:trackQuest(uid)
		end
	end
end

function module:updateQuestsTracker(event)
	LibQTip:Release(tracker)
	module:updateQuestTracker()
end

function module:isQuestTracked(uid)
	if chdb.questsTracked[uid] then
		return true
	else
		return false
	end
end

function module:trackQuest(uid)
	if core.quests[uid].id and core.quests[uid].numObjectives > 0 then
		AddQuestWatch(core.quests[uid].id)
	else
		numTrackedQuests = numTrackedQuests + 1
		chdb.questsTracked[uid] = true
		if not chdb.questsTrackedZone[core.quests[uid].zone] then
			chdb.questsTrackedZone[core.quests[uid].zone] = true
			chdb.questsTrackedByZone[core.quests[uid].zone] = {}
			chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = 0
			chdb.questsTrackedByZone[core.quests[uid].zone].quests = {}
		end
		if not chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] then
			chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] = true
			chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = chdb.questsTrackedByZone[core.quests[uid].zone].numQuests + 1
		end
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end

function module:unTrackQuest(uid)
	if core.quests[uid].id and core.quests[uid].numObjectives > 0 then
		module:RemoveQuestWatch(core.quests[uid].id)
	else
		chdb.questsTracked[uid] = nil
		chdb.questsAutoTracked[uid] = nil
		if chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] then
			chdb.questsTrackedByZone[core.quests[uid].zone].quests[uid] = nil
			chdb.questsTrackedByZone[core.quests[uid].zone].numQuests = chdb.questsTrackedByZone[core.quests[uid].zone].numQuests - 1
			if chdb.questsTrackedByZone[core.quests[uid].zone].numQuests == 0 then
				chdb.questsTrackedByZone[core.quests[uid].zone].quests = nil
				chdb.questsTrackedByZone[core.quests[uid].zone] = nil
				chdb.questsTrackedZone[core.quests[uid].zone] = nil
			end
		end
		numTrackedQuests = numTrackedQuests - 1
		LibQTip:Release(tracker)
		module:updateQuestTracker()
	end
end


----------------------------
--- Custom Cell Provider ---
----------------------------
local cProvider, cPrototype, basecCellPrototype = LibQTip:CreateCellProvider(LibQTip.LabelProvider)
function cPrototype:InitializeCell()
	self.fontString = self:CreateFontString()
	self.fontString:SetFontObject(GameTooltipText)
end

function cPrototype:SetupCell(tooltip, value, justification, font, bType, bValue, maxWidth, indent, anchor)
	self:EnableMouse(true)
	self:SetScript("OnMouseDown", function(frame, button)
		if bType == "header" then
			if chdb.collapseAll == 1 then
				chdb.collapseAll = 0
				for zone in pairs(chdb.zoneDisplay) do
					chdb.zoneDisplay[zone] = 0
				end
			else
				chdb.collapseAll = 1
				for zone in pairs(chdb.zoneDisplay) do
					chdb.zoneDisplay[zone] = 1
				end
			end
			tracker:SetScript("OnLeave", nil)
			LibQTip:Release(tracker)
			tracker = nil
			module:updateQuestTracker()
		elseif bType == "zone" then
			if chdb.zoneDisplay[bValue] == 1 then
				chdb.zoneDisplay[bValue] = 0
				chdb.collapseAll = 0
			else
				chdb.zoneDisplay[bValue] = 1
				chdb.collapseAll = 1
			end
			tracker:SetScript("OnLeave", nil)
			LibQTip:Release(tracker)
			tracker = nil
			module:updateQuestTracker()
		elseif bType == "quest" then
			if button == "RightButton" then
				if ChatFrameEditBox:IsVisible() then
					if IsShiftKeyDown() then
						if core.quests[bValue].party[core.playerName] and core.quests[bValue].id and prdb.chatlink then
							ChatEdit_InsertLink(GetQuestLink(core.quests[bValue].id))
						end
					elseif IsControlKeyDown() then
						if core.quests[bValue].numObjectives > 0 and core.quests[bValue].party[core.playerName] and prdb.questsobjectives then
							for objective in pairs(core.questsObjectives[bValue]) do
								ChatFrameEditBox:Insert(format("{%s %s/%s} ", core.questsObjectives[bValue][objective].desc, core.questsObjectives[bValue][objective].party[core.playerName].got, core.questsObjectives[bValue][objective].need))
							end
						end
					end
				elseif core.quests[bValue].party[core.playerName] and core.quests[bValue].id and prdb.clicktooltip then
					ItemRefTooltip:ClearLines()
					ShowUIPanel(ItemRefTooltip)
					if not ItemRefTooltip:IsVisible() then
						ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
					end
					local link = GetQuestLink(core.quests[bValue].id)
					if link then
						ItemRefTooltip:SetHyperlink(link)
					end
				end
			elseif button == "LeftButton" then
				if IsControlKeyDown() then
					if core.numPartyMembers > 0 and core.quests[bValue].party[core.playerName] and core.quests[bValue].id and prdb.sharequest then
						local wasSelected = GetQuestLogSelection()
						SelectQuestLogEntry(core.quests[bValue].id)
						if GetQuestLogPushable() then
							QuestLogPushQuest()
						end
						SelectQuestLogEntry(wasSelected)
					end
				elseif IsShiftKeyDown() then
					if prdb.trackquest then
						module:toggleTrackedQuest(nil, bValue)
					end
				elseif IsAltKeyDown() then
					if core.quests[bValue].party[core.playerName] and core.quests[bValue].id and prdb.abandonquest then
						local wasSelected = GetQuestLogSelection()
						SelectQuestLogEntry(core.quests[bValue].id)
						SetAbandonQuest()
						local items = GetAbandonQuestItems()
						if items then
							StaticPopup_Hide("ABANDON_QUEST")
							StaticPopup_Show("ABANDON_QUEST_WITH_ITEMS", GetAbandonQuestName(), items)
						else
							StaticPopup_Hide("ABANDON_QUEST_WITH_ITEMS")
							StaticPopup_Show("ABANDON_QUEST", GetAbandonQuestName())
						end
						SelectQuestLogEntry(wasSelected)
					end
				else
					core:displayQuestInformations(bValue)
				end
			end
		end
	end)
	self:SetScript("OnEnter", function()
		if prdb.overtooltip and bType == "quest" and core.quests[bValue].id then
			local link = GetQuestLink(core.quests[bValue].id)
			if link then
				GameTooltip:SetOwner(tooltip, anchor)
				GameTooltip:SetHyperlink(link)
			end
		end
	end)
	self:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)

	indent = indent or 0
	local width, height = basecCellPrototype.SetupCell(self, tooltip, value, justification, font, arg)
	local fs = self.fontString
	fs:SetPoint("TOPLEFT", self, "TOPLEFT", indent, 0)
	fs:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0)
	width = width + indent
	if width > maxWidth then
		fs:SetWidth(maxWidth)
		height = fs:GetHeight()
		width = maxWidth
	end
	fs:Show()
	return width, height
end

function cPrototype:ReleaseCell() 
	self:EnableMouse(false) 
	self:ClearAllPoints()
	self:SetScript("OnEnter", nil) 
	self:SetScript("OnLeave", nil) 
	self:SetScript("OnMouseDown", nil) 
end


function module:updateQuestTracker()
	if numTrackedQuests == 0 then return end
	if prdb.hideTrackerInCombat and UnitAffectingCombat("player") then return end
	local tracked = 0
	tracker = LibQTip:Acquire("tomQuest2Tracker", 2, "LEFT", "RIGHT")
	tracker:EnableMouse(true)
	tracker:SetBackdropColor(prdb.backDropColor[1], prdb.backDropColor[2], prdb.backDropColor[3], prdb.backDropColor[4])
	tracker:SetBackdropBorderColor(prdb.borderColor[1], prdb.borderColor[2], prdb.borderColor[3], prdb.borderColor[4])
	tracker:SetScale(prdb.scale)
	tracker:SetFrameStrata("BACKGROUND")

	local headerFont = CreateFont("headerFont")
	headerFont:SetFont(LSM:Fetch("font", prdb.font), prdb.headerFontSize)
	tracker:SetHeaderFont(headerFont)

	local smallFont = CreateFont("smallFont")
	smallFont:SetFont(LSM:Fetch("font", prdb.font), prdb.normalFontSize)
	tracker:SetFont(smallFont)

	local tempLine
	if prdb.dispayHeader then
		local plus =  format("|TInterface\\Buttons\\UI-PlusButton-Up:%d|t", prdb.collapseButtonSize)
		local minus = format("|TInterface\\Buttons\\UI-MinusButton-Up:%d|t", prdb.collapseButtonSize)
		for zone in pairs(chdb.questsTrackedZone) do
			tracked = tracked + 1
			chdb.zoneDisplay[zone] = chdb.zoneDisplay[zone] or 1
			tempLine = tracker:AddLine()
			tracker:SetCell(tempLine, 1, format("%s |cff%s%s|r |cff%s%s|r", chdb.zoneDisplay[zone] == 0 and plus or minus, prdb.colorizeHeaderName and core.questsZones[zone].color or "ffffff", zone, prdb.colorizeHeaderLevel and core.questsZones[zone].color or "ffffff", core.questsZones[zone].level ), headerFont, "LEFT", 2, cProvider, "zone", zone, prdb.maxWidth, 0)
			if chdb.zoneDisplay[zone] == 1 then
				for uid in pairs(chdb.questsTrackedByZone[zone].quests) do
					tempLine = tracker:AddLine()
					tracked = tracked + 1
					tracker:SetCell(tempLine, 1, format("|cff%s%s|r|cff%s%s|r %s", prdb.colorizeQuestLevel and core.quests[uid].color or "ffffff", core.quests[uid].tag, prdb.colorizeQuestName and core.quests[uid].color or "ffffff", core.quests[uid].title, core.quests[uid].playersOnQuest > 1 and format("<%d>", core.quests[uid].playersOnQuest) or ""), headerFont, "LEFT", 1, cProvider, "quest", uid, prdb.maxWidth, prdb.questscellpadding, trackerAnchor)
					tracker:SetCell(tempLine, 2, core.quests[uid].pqStatus, headerFont, "RIGHT", 1, cProvider, "quest", uid, prdb.maxWidth, 0, trackerAnchor)
					if core.questsObjectives[uid] then
						for objective in pairs(core.questsObjectives[uid]) do
							if prdb.hideCompletedObjective == 3 or (prdb.hideCompletedObjective == 2 and not core.questsObjectives[uid][objective].partyComplete) or (prdb.hideCompletedObjective == 1 and not core.questsObjectives[uid][objective].playerComplete) then
								tempLine = tracker:AddLine()
								tracker:SetCell(tempLine, 1, format("|cff%s%s|r", prdb.colorizeQuestObjective and core.questsObjectives[uid][objective].qoColor or "ffffff", core.questsObjectives[uid][objective].desc), smallFont, "LEFT", 1, cProvider, "quest", uid, prdb.maxWidth, prdb.objectivescellpadding, trackerAnchor)
								tracker:SetCell(tempLine, 2, core.questsObjectives[uid][objective].qoStatus, smallFont, "RIGHT", 1, cProvider, "quest", uid, prdb.maxWidth, 0, trackerAnchor)
							end
						end
					else
						tempLine = tracker:AddLine()
						tracker:SetCell(tempLine, 1, core.quests[uid].objective, smallFont, "LEFT", 2, cProvider,"quest", uid, prdb.maxWidth, prdb.objectivescellpadding, trackerAnchor)
					end
				end
			end
		end
	else
		for uid in pairs(chdb.questsTracked) do
			if uid and core.quests[uid] then
				if prdb.hideCompletedQuest == 3 or (prdb.hideCompletedQuest == 2 and not core.quests[uid].partyComplete) or (prdb.hideCompletedQuest == 1 and not core.quests[uid].playerComplete) then
					tempLine = tracker:AddLine()
					tracked = tracked + 1
					tracker:SetCell(tempLine, 1, format("|cff%s%s|r|cff%s%s|r %s", prdb.colorizeQuestLevel and core.quests[uid].color or "ffffff", core.quests[uid].tag, prdb.colorizeQuestName and core.quests[uid].color or "ffffff", core.quests[uid].title, core.quests[uid].playersOnQuest > 1 and format("<%d>", core.quests[uid].playersOnQuest) or ""), headerFont, "LEFT", 1, cProvider, "quest", uid, prdb.maxWidth, 0, trackerAnchor)
					tracker:SetCell(tempLine, 2, core.quests[uid].pqStatus, headerFont, "RIGHT", 1, cProvider,  "quests",uid, prdb.maxWidth, 0, trackerAnchor)
					if core.questsObjectives[uid] then
						for objective in pairs(core.questsObjectives[uid]) do
							if prdb.hideCompletedObjective == 3 or (prdb.hideCompletedObjective == 2 and not core.questsObjectives[uid][objective].partyComplete) or (prdb.hideCompletedObjective == 1 and not core.questsObjectives[uid][objective].playerComplete) then
								tempLine = tracker:AddLine()
								tracker:SetCell(tempLine, 1, format("|cff%s%s|r", prdb.colorizeQuestObjective and core.questsObjectives[uid][objective].qoColor or "ffffff", core.questsObjectives[uid][objective].desc), smallFont, "LEFT", 1, cProvider, "quest", uid, prdb.maxWidth, prdb.objectivescellpadding, trackerAnchor)
								tracker:SetCell(tempLine, 2, core.questsObjectives[uid][objective].qoStatus, smallFont, "RIGHT", 1, cProvider, "quest", uid, prdb.maxWidth, 0, trackerAnchor)
							end
						end
					else
						tempLine = tracker:AddLine()
						tracker:SetCell(tempLine, 1, core.quests[uid].objective, smallFont, "LEFT", 2, cProvider, "quest", uid, prdb.maxWidth, prdb.objectivescellpadding, trackerAnchor)
					end
				end
			end
		end
	end
	if tracked > 0 then
		tracker:SetClampedToScreen(true)
		tracker:ClearAllPoints()
		if prdb.anchorPosition == 1 then
			if prdb.showAnchor then
				tracker:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT")
			else
				tracker:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, 24)
			end
		elseif prdb.anchorPosition == 2 then
			if prdb.showAnchor then
				tracker:SetPoint("TOP", anchor, "BOTTOM")
			else
				tracker:SetPoint("TOP", anchor, "BOTTOM", 0, 24)
			end
		elseif prdb.anchorPosition == 3 then
			if prdb.showAnchor then
				tracker:SetPoint("TOPRIGHT", anchor, "BOTTOMRIGHT")
			else
				tracker:SetPoint("TOPRIGHT", anchor, "BOTTOMRIGHT", 0, 24)
			end
		elseif prdb.anchorPosition == 4 then
			if prdb.showAnchor then
				tracker:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT")
			else
				tracker:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 0, -24)
			end
		elseif prdb.anchorPosition == 5 then
			if prdb.showAnchor then
				tracker:SetPoint("BOTTOM", anchor, "TOP")
			else
				tracker:SetPoint("BOTTOM", anchor, "TOP", 0, -24)
			end
		elseif prdb.anchorPosition == 6 then
			if prdb.showAnchor then
				tracker:SetPoint("BOTTOMRIGHT", anchor, "TOPRIGHT")
			else
				tracker:SetPoint("BOTTOMRIGHT", anchor, "TOPRIGHT", 0, -24)
			end
		end
		tracker:Show()
	else
		LibQTip:Release(tracker)
	end
end